@@ -21,20 +21,13 @@
 #
 # ---------------------------------------------------------------------------
 
-# imports
-
-# PySol imports
-from pysollib.gamedb import registerGame, GameInfo, GI
-from pysollib.mfxutil import kwdefault
 from pysollib.game import Game
-from pysollib.layout import Layout
-from pysollib.hint import DefaultHint
-from pysollib.hint import FreeCellType_Hint, FreeCellSolverWrapper
-
+from pysollib.gamedb import GI, GameInfo, registerGame
 from pysollib.games.spider import Spider_AC_Foundation
-
-from pysollib.util import ACE, ANY_SUIT, KING, NO_RANK, UNLIMITED_CARDS
-
+from pysollib.hint import DefaultHint
+from pysollib.hint import FreeCellSolverWrapper, FreeCellType_Hint
+from pysollib.layout import Layout
+from pysollib.mfxutil import kwdefault
 from pysollib.stack import \
         AC_FoundationStack, \
         AC_RowStack, \
@@ -48,10 +41,11 @@ from pysollib.stack import \
         ReserveStack, \
         SS_FoundationStack, \
         Stack, \
+        StackWrapper, \
         SuperMoveAC_RowStack, \
         SuperMoveRK_RowStack, \
-        isAlternateColorSequence, \
-        StackWrapper
+        isAlternateColorSequence
+from pysollib.util import ACE, ANY_SUIT, KING, NO_RANK, UNLIMITED_CARDS
 
 # ************************************************************************
 # * FreeCell
@@ -273,6 +267,17 @@ class TripleFreecell(FreeCell):
         self._startDealNumRowsAndDealSingleRow(11)
 
 
+class DoubleFreecellTd(TripleFreecell):
+    def createGame(self):
+        TripleFreecell.createGame(self, reserves=8, rows=10, playcards=20)
+
+    def startGame(self):
+        self._startDealNumRows(9)
+        self.s.talon.dealRow()
+        r = self.s.rows
+        self.s.talon.dealRow(rows=r[:4])
+
+
 class Cell11(TripleFreecell):
     def createGame(self):
         TripleFreecell.createGame(self, rows=12, reserves=11)
@@ -644,13 +649,21 @@ class PairFcFreeCell(FreeCell):
         FreeCell.createGame(self, reserves=2)
 
 
+class ZeroFcFreeCell(FreeCell):
+    def createGame(self):
+        FreeCell.createGame(self, reserves=0)
+
+
 # register the game
 registerGame(GameInfo(5, RelaxedFreeCell, "Relaxed FreeCell",
                       GI.GT_FREECELL | GI.GT_RELAXED | GI.GT_OPEN, 1, 0,
                       GI.SL_SKILL))
 registerGame(GameInfo(8, FreeCell, "FreeCell",
+                      GI.GT_FREECELL | GI.GT_OPEN, 1, 0, GI.SL_SKILL,
+                      altnames=("Four Free")))
+registerGame(GameInfo(1901, ZeroFcFreeCell, "FreeCell (No Reserve)",
                       GI.GT_FREECELL | GI.GT_OPEN, 1, 0, GI.SL_SKILL))
-registerGame(GameInfo(1900, PairFcFreeCell, "FreeCell with Two Reserves",
+registerGame(GameInfo(1900, PairFcFreeCell, "FreeCell (2 Reserves)",
                       GI.GT_FREECELL | GI.GT_OPEN, 1, 0, GI.SL_SKILL))
 registerGame(GameInfo(46, ForeCell, "ForeCell",
                       GI.GT_FREECELL | GI.GT_OPEN, 1, 0, GI.SL_MOSTLY_SKILL))
@@ -662,8 +675,7 @@ registerGame(GameInfo(264, DoubleFreecell, "Double FreeCell",
 registerGame(GameInfo(265, TripleFreecell, "Triple FreeCell",
                       GI.GT_FREECELL | GI.GT_OPEN, 3, 0, GI.SL_MOSTLY_SKILL))
 registerGame(GameInfo(336, ChallengeFreeCell, "Challenge FreeCell",
-                      GI.GT_FREECELL | GI.GT_OPEN, 1, 0, GI.SL_SKILL,
-                      rules_filename='freecell.html'))
+                      GI.GT_FREECELL | GI.GT_OPEN, 1, 0, GI.SL_SKILL))
 registerGame(GameInfo(337, SuperChallengeFreeCell, "Super Challenge FreeCell",
                       GI.GT_FREECELL | GI.GT_OPEN, 1, 0, GI.SL_SKILL))
 registerGame(GameInfo(363, Spidercells, "Spidercells",
@@ -686,9 +698,9 @@ registerGame(GameInfo(464, FourColours, "Four Colours",
 registerGame(GameInfo(509, BigCell, "Big Cell",
                       GI.GT_FREECELL | GI.GT_OPEN | GI.GT_ORIGINAL, 3, 0,
                       GI.SL_MOSTLY_SKILL))
-registerGame(GameInfo(513, OceanTowers, "Ocean Towers",
-                      GI.GT_FREECELL | GI.GT_OPEN | GI.GT_ORIGINAL, 2, 0,
-                      GI.SL_MOSTLY_SKILL))
+registerGame(GameInfo(513, OceanTowers, "Double Seahaven Towers",
+                      GI.GT_FREECELL | GI.GT_OPEN, 2, 0, GI.SL_MOSTLY_SKILL,
+                      altnames=("Ocean Towers")))
 registerGame(GameInfo(520, GermanFreeCell, "German FreeCell",
                       GI.GT_FREECELL | GI.GT_OPEN, 1, 0, GI.SL_SKILL))
 registerGame(GameInfo(542, KingCell, "KingCell",
@@ -699,5 +711,7 @@ registerGame(GameInfo(648, Headquarters, "Headquarters",
 registerGame(GameInfo(698, CanCan, "Can Can",
                       GI.GT_RAGLAN | GI.GT_OPEN, 1, 0, GI.SL_MOSTLY_SKILL))
 registerGame(GameInfo(746, Limpopo, "Limpopo",
-                      GI.GT_FREECELL | GI.GT_ORIGINAL, 2, 0,
+                      GI.GT_FREECELL | GI.GT_OPEN | GI.GT_ORIGINAL, 2, 0,
                       GI.SL_MOSTLY_SKILL))
+registerGame(GameInfo(813, DoubleFreecellTd, "Double FreeCell (Traditional)",
+                      GI.GT_FREECELL | GI.GT_OPEN, 2, 0, GI.SL_MOSTLY_SKILL))
