@@ -154,10 +154,10 @@ class GrandDuchessPlus(GrandDuchess):
 
 
 registerGame(GameInfo(557, GrandDuchess, "Grand Duchess",
-                      GI.GT_2DECK_TYPE, 2, 3))
+                      GI.GT_2DECK_TYPE, 2, 3,
+                      altnames=('Duchess de Luynes')))
 registerGame(GameInfo(617, Parisienne, "Parisienne",
                       GI.GT_2DECK_TYPE, 2, 3,
-                      rules_filename='grandduchess.html',
                       altnames=('La Parisienne', 'Parisian')))
 registerGame(GameInfo(618, GrandDuchessPlus, "Grand Duchess +",
                       GI.GT_2DECK_TYPE, 2, 3))
