@@ -1,42 +1,46 @@
 #!/usr/bin/env python
 # -*- mode: python; coding: utf-8; -*-
-##---------------------------------------------------------------------------##
-##
-## Copyright (C) 1998-2003 Markus Franz Xaver Johannes Oberhumer
-## Copyright (C) 2003 Mt. Hood Playing Card Co.
-## Copyright (C) 2005-2009 Skomoroh
-##
-## This program is free software: you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation, either version 3 of the License, or
-## (at your option) any later version.
-##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
-##
-##---------------------------------------------------------------------------##
+# ---------------------------------------------------------------------------##
+#
+# Copyright (C) 1998-2003 Markus Franz Xaver Johannes Oberhumer
+# Copyright (C) 2003 Mt. Hood Playing Card Co.
+# Copyright (C) 2005-2009 Skomoroh
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# ---------------------------------------------------------------------------##
 
-__all__ = []
-
-# imports
-import sys
-
-# PySol imports
-from pysollib.gamedb import registerGame, GameInfo, GI
-from pysollib.util import *
-from pysollib.stack import *
 from pysollib.game import Game
+from pysollib.gamedb import GI, GameInfo, registerGame
+from pysollib.hint import CautiousDefaultHint, DefaultHint
 from pysollib.layout import Layout
-from pysollib.hint import AbstractHint, DefaultHint, CautiousDefaultHint
+from pysollib.pysoltk import MfxCanvasText
+from pysollib.stack import \
+    AC_FoundationStack, \
+    BasicRowStack, \
+    DealRowTalonStack, \
+    InitialDealTalonStack, \
+    InvisibleStack, \
+    RK_RowStack, \
+    SC_RowStack, \
+    SS_FoundationStack, \
+    SS_RowStack, \
+    StackWrapper, \
+    WasteStack, \
+    WasteTalonStack
+from pysollib.util import ACE, ANY_SUIT, BLACK, JACK, KING, QUEEN, RANKS, RED
 
-# ************************************************************************
-# *
-# ************************************************************************
 
 class GrandfathersClock_Hint(CautiousDefaultHint):
     # FIXME: demo is not too clever in this game
@@ -63,31 +67,33 @@ class GrandfathersClock(Game):
 
         # set window
         # (piles up to 9 cards are fully playable in default window size)
-        dh = max(3*l.YS/2+l.CH, l.YS+(9-1)*l.YOFFSET)
+        dh = max(3*l.YS//2+l.CH, l.YS+(9-1)*l.YOFFSET)
         self.setSize(10*l.XS+l.XM, l.YM+2*dh)
 
         # create stacks
         for i in range(2):
             x, y = l.XM, l.YM + i*dh
             for j in range(4):
-                s.rows.append(RK_RowStack(x, y, self, max_move=1, max_accept=1))
+                s.rows.append(
+                    RK_RowStack(x, y, self, max_move=1, max_accept=1))
                 x = x + l.XS
-        y = l.YM + dh - l.CH / 2
-        self.setRegion(s.rows[:4], (-999, -999, x - l.XM / 2, y))
-        self.setRegion(s.rows[4:], (-999,    y, x - l.XM / 2, 999999))
-        d = [ (0,0), (1,0.15), (2,0.5), (2.5,1.5), (2,2.5), (1,2.85) ]
+        y = l.YM + dh - l.CH // 2
+        self.setRegion(s.rows[:4], (-999, -999, x - l.XM // 2, y))
+        self.setRegion(s.rows[4:], (-999,    y, x - l.XM // 2, 999999))
+        d = [(0, 0), (1, 0.15), (2, 0.5), (2.5, 1.5), (2, 2.5), (1, 2.85)]
         for i in range(len(d)):
-            d.append( (0 - d[i][0], 3 - d[i][1]) )
+            d.append((0 - d[i][0], 3 - d[i][1]))
         x0, y0 = l.XM, l.YM + dh - l.CH
         for i in range(12):
             j = (i + 5) % 12
-            x = int(round(x0 + ( 6.5+d[j][0]) * l.XS))
+            x = int(round(x0 + (6.5+d[j][0]) * l.XS))
             y = int(round(y0 + (-1.5+d[j][1]) * l.YS))
-            suit = (1, 2, 0, 3) [i % 4]
+            suit = (1, 2, 0, 3)[i % 4]
             s.foundations.append(SS_FoundationStack(x, y, self, suit,
                                                     base_rank=i+1, mod=13,
                                                     max_move=0))
-        s.talon = InitialDealTalonStack(self.width-l.XS, self.height-l.YS, self)
+        s.talon = InitialDealTalonStack(
+            self.width-l.XS, self.height-l.YS, self)
 
         # define stack-groups
         self.sg.openstacks = s.foundations + s.rows
@@ -107,13 +113,13 @@ class GrandfathersClock(Game):
                 clocks.append(c)
                 cards.remove(c)
         # sort clocks reverse by rank
-        clocks.sort(lambda a, b: cmp(b.rank, a.rank))
+        clocks.sort(key=lambda x: -x.rank)
         return clocks + cards
 
     def startGame(self):
-        self.playSample("grandfathersclock", loop=1)
-        for i in range(4):
-            self.s.talon.dealRow(frames=0)
+        self.startDealSample()
+        self.playSample("grandfathersclock", loop=1, priority=200)
+        self._dealNumRows(4)
         self.s.talon.dealRow()
         self.s.talon.dealRow(rows=self.s.foundations)
 
@@ -139,6 +145,8 @@ class Dial(Game):
 
         x0, y0 = l.XM+2*l.XS, l.YM
         rank = 0
+        font = self.app.getFont("canvas_default")
+
         for xx, yy in ((3.5, 0.15),
                        (4.5, 0.5),
                        (5,   1.5),
@@ -155,17 +163,30 @@ class Dial(Game):
                        ):
             x = int(x0 + xx*l.XS)
             y = int(y0 + yy*l.YS)
-            s.foundations.append(AC_FoundationStack(x, y, self, suit=ANY_SUIT,
-                                 dir=0, max_cards=4, base_rank=rank, max_move=0))
+            stack = AC_FoundationStack(
+                    x, y, self, suit=ANY_SUIT,
+                    dir=0, max_cards=4, base_rank=rank, max_move=0)
+            stack.getBottomImage = stack._getReserveBottomImage
+            s.foundations.append(stack)
+            if self.preview <= 1:
+                label = RANKS[rank][0]
+                if label == "1":
+                    label = "10"
+                stack.texts.misc = MfxCanvasText(self.canvas,
+                                                 x + l.CW // 2,
+                                                 y + l.CH // 2,
+                                                 anchor="center",
+                                                 font=font)
+                stack.texts.misc.config(text=label)
             rank += 1
 
         x, y = l.XM, l.YM
         s.talon = WasteTalonStack(x, y, self, max_rounds=2)
         l.createText(s.talon, 's')
-        l.createRoundText(s.talon, 'sss')
         x += l.XS
         s.waste = WasteStack(x, y, self)
         l.createText(s.waste, 's')
+        l.createRoundText(s.talon, 'ne', dx=l.XS)
 
         l.defaultStackGroups()
 
@@ -178,9 +199,6 @@ class Dial(Game):
 # * Hemispheres
 # ************************************************************************
 
-BLACK, RED = 0, 1
-
-
 class Hemispheres_Hint(DefaultHint):
     def shallMovePile(self, from_stack, to_stack, pile, rpile):
         if not self._defaultShallMovePile(from_stack, to_stack, pile, rpile):
@@ -295,12 +313,12 @@ class Hemispheres(Game):
         # foundations
         x, y = x0+2*l.XS, y0+1.5*l.YS
         for i in range(4):
-            s.foundations.append(SS_FoundationStack(x, y, self, suit=2+i/2,
+            s.foundations.append(SS_FoundationStack(x, y, self, suit=2+i//2,
                                                     max_move=0))
             x += l.XS
         x, y = x0+2*l.XS, y+l.YS
         for i in range(4):
-            s.foundations.append(SS_FoundationStack(x, y, self, suit=i/2,
+            s.foundations.append(SS_FoundationStack(x, y, self, suit=i//2,
                                  max_move=0, base_rank=KING, dir=-1))
             x += l.XS
 
@@ -314,30 +332,26 @@ class Hemispheres(Game):
 
         l.defaultStackGroups()
 
-
     def _shuffleHook(self, cards):
         founds_cards = []               # foundations
         rows_cards = []                 # rows
         for c in cards[:]:
             if c.rank in (ACE, KING):
-                if ((c.rank == ACE and c.color == RED) or
-                    (c.rank == KING and c.color == BLACK)):
+                cond = ((c.rank == ACE and c.color == RED) or
+                        (c.rank == KING and c.color == BLACK))
+                if cond:
                     cards.remove(c)
                     founds_cards.append(c)
                 elif c.deck == 0:
                     cards.remove(c)
                     rows_cards.append(c)
-        founds_cards.sort(lambda a, b: cmp((-a.rank, -a.suit), (-b.rank, -b.suit)))
-        rows_cards.sort(lambda a, b: cmp((a.rank, a.suit), (b.rank, b.suit)))
+        founds_cards.sort(key=lambda x: (-x.rank, -x.suit))
+        rows_cards.sort(key=lambda x: (x.rank, x.suit))
         return cards+rows_cards+founds_cards
 
-
     def startGame(self):
         self.s.talon.dealRow(rows=self.s.foundations, frames=0)
-        self.startDealSample()
-        self.s.talon.dealRow()
-        self.s.talon.dealCards()       # deal first card to WasteStack
-
+        self._startAndDealRowAndCards()
 
     def fillStack(self, stack):
         if stack in self.s.rows[4:] and not stack.cards:
@@ -348,7 +362,6 @@ class Hemispheres(Game):
                 self.s.waste.moveMove(1, stack)
             self.leaveState(old_state)
 
-
     def shallHighlightMatch(self, stack1, card1, stack2, card2):
         # by color
         return card1.color == card2.color and abs(card1.rank-card2.rank) == 1
@@ -384,6 +397,7 @@ class BigBen_Talon(DealRowTalonStack):
             self.game.stopSamples()
         return ncards
 
+
 class BigBen_RowStack(SS_RowStack):
     def acceptsCards(self, from_stack, cards):
         if not SS_RowStack.acceptsCards(self, from_stack, cards):
@@ -392,6 +406,7 @@ class BigBen_RowStack(SS_RowStack):
             return False
         return True
 
+
 class BigBen(Game):
     Hint_Class = CautiousDefaultHint
 
@@ -422,10 +437,10 @@ class BigBen(Game):
             (2.5, 3),
             (1.5, 2.85),
             (0.5, 2.5),
-            ):
+                ):
             x = int(x0 + xx*l.XS)
             y = int(y0 + yy*l.YS)
-            suit=(3,0,2,1)[rank%4]
+            suit = (3, 0, 2, 1)[rank % 4]
             max_cards = rank <= 4 and 8 or 9
             s.foundations.append(SS_FoundationStack(x, y, self, suit=suit,
                                  max_cards=max_cards, base_rank=rank,
@@ -443,9 +458,9 @@ class BigBen(Game):
 
     def _shuffleHook(self, cards):
         # move clock cards to top of the Talon (i.e. first cards to be dealt)
-        C, S, H, D = range(4)           # suits
-        t = [(1,C), (2,H), (3,S), (4,D), (5,C), (6,H),
-             (7,S), (8,D), (9,C), (JACK,H), (QUEEN,S), (KING,D)]
+        C, S, H, D = list(range(4))           # suits
+        t = [(1, C), (2, H), (3, S), (4, D), (5, C), (6, H),
+             (7, S), (8, D), (9, C), (JACK, H), (QUEEN, S), (KING, D)]
         clocks = []
         for c in cards[:]:
             if (c.rank, c.suit) in t:
@@ -455,7 +470,7 @@ class BigBen(Game):
             if not t:
                 break
         # sort clocks reverse by rank
-        clocks.sort(lambda a, b: cmp(b.rank, a.rank))
+        clocks.sort(key=lambda x: -x.rank)
         return cards+clocks
 
     def startGame(self):
@@ -476,6 +491,7 @@ class BigBen(Game):
 # ************************************************************************
 
 class Clock_RowStack(RK_RowStack):
+    getBottomImage = RK_RowStack._getReserveBottomImage
 
     def _numFaceDown(self):
         ncards = 0
@@ -535,6 +551,10 @@ class Clock_RowStack(RK_RowStack):
 
 
 class Clock(Game):
+    RowStack_Class = Clock_RowStack
+    Talon_Class = InitialDealTalonStack
+
+    HAS_WASTE = False
 
     def createGame(self):
         # create layout
@@ -543,9 +563,14 @@ class Clock(Game):
         # set window
         dx = l.XS + 3*l.XOFFSET
         w = max(5.25*dx + l.XS, 5.5*dx)
+        if self.HAS_WASTE:
+            w += (1.5 * l.XS)
         self.setSize(l.XM + w, l.YM + 4*l.YS)
 
+        font = self.app.getFont("canvas_default")
+
         # create stacks
+        row_rank = 0
         for xx, yy in (
             (3.25, 0.15),
             (4.25, 0.5),
@@ -559,29 +584,61 @@ class Clock(Game):
             (0.25, 0.5),
             (1.25, 0.15),
             (2.25, 0),
-            ):
+                ):
             x = l.XM + xx*dx
             y = l.YM + yy*l.YS
-            stack = Clock_RowStack(x, y, self, max_move=0)
+            if self.HAS_WASTE:
+                x += (2 * l.XS)
+            stack = self.RowStack_Class(x, y, self, max_move=0,
+                                        base_rank=row_rank)
             stack.CARD_XOFFSET, stack.CARD_YOFFSET = l.XOFFSET, 0
             stack.SHRINK_FACTOR = 1
             s.rows.append(stack)
+            if self.preview <= 1:
+                label = RANKS[row_rank][0]
+                if label == "1":
+                    label = "10"
+                stack.texts.misc = MfxCanvasText(self.canvas,
+                                                 x + l.CW // 2,
+                                                 y + l.CH // 2,
+                                                 anchor="center",
+                                                 font=font)
+                stack.texts.misc.config(text=label)
+            row_rank += 1
 
         x, y = l.XM + 2.25*dx, l.YM + 1.5*l.YS
-        stack = Clock_RowStack(x, y, self, max_move=1)
+        if self.HAS_WASTE:
+            x += (2 * l.XS)
+        stack = self.RowStack_Class(x, y, self, max_move=1, base_rank=row_rank)
         stack.CARD_XOFFSET, stack.CARD_YOFFSET = l.XOFFSET, 0
         stack.SHRINK_FACTOR = 1
         s.rows.append(stack)
+        if self.preview <= 1:
+            stack.texts.misc = MfxCanvasText(self.canvas,
+                                             x + l.CW // 2,
+                                             y + l.CH // 2,
+                                             anchor="center",
+                                             font=font)
+            stack.texts.misc.config(text=(RANKS[row_rank][0]))
 
-        x, y = self.width - l.XS, self.height - l.YS
-        s.talon = InitialDealTalonStack(x, y, self)
+        if self.HAS_WASTE:
+            x, y = l.XM, l.YM
+            s.talon = self.Talon_Class(x, y, self)
+            l.createText(s.talon, 's')
+            x += l.XS
+            s.waste = WasteStack(x, y, self)
+            l.createText(s.waste, 's')
+            l.createRoundText(s.talon, 'ne', dx=l.XS)
+        else:
+            x, y = self.width - l.XS, self.height - l.YS
+            s.talon = self.Talon_Class(x, y, self)
 
         # create an invisible stacks
         s.internals.append(InvisibleStack(self))
         s.internals.append(InvisibleStack(self))
 
         # default
-        l.defaultAll()
+        l.defaultStackGroups()
 
     def startGame(self):
         for i in range(3):
@@ -604,6 +661,43 @@ class Clock(Game):
         return (), (), ()
 
 
+# ************************************************************************
+# * German Clock
+# ************************************************************************
+
+class GermanClock_RowStack(AC_FoundationStack):
+    getBottomImage = AC_FoundationStack._getReserveBottomImage
+
+    def acceptsCards(self, from_stack, cards):
+        num_cards = len(self.cards)
+        for i in range(13):
+            check_seq = self.game.s.rows[i].cards
+            if len(check_seq) > num_cards:
+                if check_seq[num_cards].suit != cards[0].suit:
+                    return False
+
+        return AC_FoundationStack.acceptsCards(self, from_stack, cards)
+
+
+class GermanClock(Clock):
+    RowStack_Class = StackWrapper(GermanClock_RowStack, dir=0, max_move=0,
+                                  suit=ANY_SUIT)
+    Talon_Class = StackWrapper(WasteTalonStack, max_rounds=2)
+
+    HAS_WASTE = True
+
+    def startGame(self):
+        pass
+
+    def isGameWon(self):
+        for r in self.s.rows:
+            if len(r.cards) < 4:
+                return False
+        return True
+
+    def getAutoStacks(self, event=None):
+        return Game.getAutoStacks(self, event)
+
 
 # register the game
 registerGame(GameInfo(261, GrandfathersClock, "Grandfather's Clock",
@@ -612,10 +706,12 @@ registerGame(GameInfo(682, Dial, "Dial",
                       GI.GT_1DECK_TYPE, 1, 1, GI.SL_LUCK))
 registerGame(GameInfo(690, Hemispheres, "Hemispheres",
                       GI.GT_2DECK_TYPE, 2, 0, GI.SL_BALANCED,
-                      altnames=("The Four Continents",) ))
+                      altnames=("The Four Continents",)))
 registerGame(GameInfo(697, BigBen, "Big Ben",
                       GI.GT_2DECK_TYPE, 2, 0, GI.SL_BALANCED))
 registerGame(GameInfo(737, Clock, "Clock",
                       GI.GT_1DECK_TYPE, 1, 0, GI.SL_LUCK,
-                      altnames=("Travellers",) ))
-
+                      altnames=("Travellers", "Sundial")))
+registerGame(GameInfo(827, GermanClock, "German Clock",
+                      GI.GT_1DECK_TYPE, 1, 1, GI.SL_MOSTLY_LUCK,
+                      altnames=("Die Uhr",)))
