@@ -23,23 +23,17 @@
 
 import math
 
-from pysollib.mygettext import _
-from pysollib.mfxutil import kwdefault
 from pysollib.game import Game
 from pysollib.hint import DefaultHint
-
-from pysollib.util import ANY_RANK, ANY_SUIT
-
+from pysollib.mfxutil import kwdefault
+from pysollib.mygettext import _
 from pysollib.stack import \
         AbstractFoundationStack, \
         OpenStack, \
         ReserveStack, \
-        isRankSequence, \
-        cardsFaceUp
-
-# ************************************************************************
-#  *
-#  ***********************************************************************/
+        cardsFaceUp, \
+        isRankSequence
+from pysollib.util import ANY_RANK, ANY_SUIT
 
 
 class AbstractFlowerGame(Game):
@@ -292,6 +286,9 @@ class Hanafuda_SequenceStack(Flower_OpenStack):
             return cards[0].rank == 0 or self.cap.base_rank == ANY_RANK
         return self.isHanafudaSequence([stackcards[-1], cards[0]])
 
+    def canMoveCards(self, cards):
+        return self.basicCanMoveCards(cards) and self.isHanafudaSequence(cards)
+
 
 class Oonsoo_SequenceStack(Flower_OpenStack):
 
@@ -304,6 +301,10 @@ class Oonsoo_SequenceStack(Flower_OpenStack):
             return cards[0].rank == 0 or self.cap.base_rank == ANY_RANK
         return self.isHanafudaSequence([stackcards[-1], cards[0]], 0)
 
+    def canMoveCards(self, cards):
+        return (self.basicCanMoveCards(cards) and
+                self.isHanafudaSequence(cards, 0))
+
 
 class FlowerClock_RowStack(Flower_OpenStack):
 
