@@ -31,6 +31,7 @@ from kivy.base import EventLoop
 from kivy.base import stopTouchApp
 from kivy.cache import Cache
 from kivy.clock import Clock
+from kivy.config import Config
 from kivy.core.audio import SoundLoader
 from kivy.core.window import Window
 from kivy.graphics import Color
@@ -43,7 +44,6 @@ from kivy.uix.actionbar import ActionPrevious
 from kivy.uix.actionbar import ActionView
 from kivy.uix.behaviors import ButtonBehavior
 from kivy.uix.boxlayout import BoxLayout
-from kivy.uix.button import Button
 from kivy.uix.image import Image as KivyImage
 from kivy.uix.label import Label
 from kivy.uix.scrollview import ScrollView
@@ -52,6 +52,12 @@ from kivy.uix.treeview import TreeViewLabel
 from kivy.uix.widget import Widget
 from kivy.utils import platform
 
+from pysollib.kivy.androidperms import requestStoragePerm
+from pysollib.resource import CSI
+
+if platform != 'android':
+    Config.set('input', 'mouse', 'mouse,multitouch_on_demand')
+
 # =============================================================================
 
 
@@ -327,6 +333,50 @@ def LColorToKivy(outline):
 # =============================================================================
 
 
+def cardfactor(canvas):
+    # heuristic to find some sort of 'fontsize' out of the cardset.
+    # We take the original small cardsets as reference and calculate
+    # a correction factor.
+
+    def pyth(a, b):
+        return math.sqrt(a*a+b*b)
+
+    cardscale = 1.0
+    try:
+        cs = canvas.wmain.app.images.cs
+        print('Cardset:', cs)
+        print('Cardset:', cs.type)
+
+        cardbase = pyth(73, 97)
+        if cs.type == CSI.TYPE_FRENCH:
+            cardbase = pyth(73, 97)
+        elif cs.type == CSI.TYPE_HANAFUDA:
+            cardbase = pyth(64, 102)
+        elif cs.type == CSI.TYPE_MAHJONGG:
+            cardbase = pyth(38, 54)
+        elif cs.type == CSI.TYPE_TAROCK:
+            cardbase = pyth(80, 80)
+        elif cs.type == CSI.TYPE_HEXADECK:
+            cardbase = pyth(50, 80)
+        elif cs.type == CSI.TYPE_MUGHAL_GANJIFA:
+            cardbase = pyth(80, 80)
+        elif cs.type == CSI.TYPE_NAVAGRAHA_GANJIFA:
+            cardbase = pyth(80, 80)
+        elif cs.type == CSI.TYPE_DASHAVATARA_GANJIFA:
+            cardbase = pyth(80, 80)
+        elif cs.type == CSI.TYPE_TRUMP_ONLY:
+            cardbase = pyth(35, 35)
+
+        si = canvas.wmain.app.images.getSize()
+        cardsize = pyth(si[0], si[1])
+        cardscale = cardsize/cardbase
+    except:	 # noqa: E722
+        pass
+    return cardscale
+
+# =============================================================================
+
+
 class LText(Widget, LBase):
     text = StringProperty('')
 
@@ -356,7 +406,7 @@ class LText(Widget, LBase):
         # print('LText: text = %s' % (self.text))
 
         kwargs['font'] = font
-        kwargs['font_size'] = fontsize
+        kwargs['font_size'] = fontsize * cardfactor(canvas)
 
         class MyLabel(Label, LBase):
             pass
@@ -647,9 +697,10 @@ class LRectangle(Widget, LBase):
                 bpos[0], tpos[1],
                 bpos[0], bpos[1]]
 
+        cf = cardfactor(self.prnt)
         dmy, brd = self.prnt.CoreToKivy(
             (0.0, 0.0), (self.border, self.border))
-        border = brd[1]
+        border = brd[1] * cf
 
         self.canvas.clear()
         with self.canvas:
@@ -671,7 +722,10 @@ class LRectangle(Widget, LBase):
                     event = LEvent()
                     event.x = ppos[0]
                     event.y = ppos[1]
-                    self.group.bindings['<1>'](event)
+                    if touch.is_double_tap:
+                        self.group.bindings['<Double-1>'](event)
+                    else:
+                        self.group.bindings['<1>'](event)
                     return True
         return False
 
@@ -708,9 +762,27 @@ class LImageItem(BoxLayout, LBase):
         # ev. noch globales cache für stacks->game und cards->stack
         # einrichten. Aber: stacks hängt vom jeweiligen spiel ab.
 
-    def send_event_pressed_1(self, event):
-        if self.group and '<1>' in self.group.bindings:
-            self.group.bindings['<1>'](event)
+    def send_event_pressed_n(self, event, n):
+        if self.group and n in self.group.bindings:
+            self.group.bindings[n](event)
+
+    def send_event_pressed(self, touch, event):
+
+        if touch.is_double_tap:
+            self.send_event_pressed_n(event, '<Double-1>')
+        else:
+            button = 'left'
+            if 'button' in touch.profile:
+                button = touch.button
+            if button == 'left':
+                self.send_event_pressed_n(event, '<1>')
+                return
+            if button == 'middle':
+                self.send_event_pressed_n(event, '<2>')
+                return
+            if button == 'right':
+                self.send_event_pressed_n(event, '<3>')
+                return
 
     def on_touch_down(self, touch):
 
@@ -734,7 +806,7 @@ class LImageItem(BoxLayout, LBase):
                                 event.y = ppos[1]
                                 self.dragstart = touch.pos
                                 event.cardid = i
-                                self.send_event_pressed_1(event)
+                                self.send_event_pressed(touch, event)
                                 return True
 
             if self.group is not None:
@@ -1168,7 +1240,7 @@ class LMenu(ActionView, LBase):
         class MyActionPrev(ActionPrevious, LBase):
             pass
 
-        kw['app_icon'] = 'data/images/misc/pysol01.png'
+        kw['app_icon'] = 'data/images/icons/48x48/pysol.png'
         kw['with_previous'] = prev
         kw['size_hint'] = (.01, 1)
         self.ap = MyActionPrev(**kw)
@@ -1522,25 +1594,20 @@ class LMainWindow(BoxLayout, LTkBase):
         self.toolBarPos = 0
         self.bindings = {}
         self._w = '.'
-        self.topLine = Button(
-            size_hint=(1.0, 0.01),
-            background_down='atlas:'
-                            '//data/images/defaulttheme/action_item_down',
-            background_normal='atlas:'
-                              '//data/images/defaulttheme/action_item',
-            border=(0, 0, 0, 0))
-        self.topLine1 = Label(size_hint=(1.0, 0.01))
 
-        self.add_widget(self.topLine)
         self.add_widget(self.menuArea)
-        self.add_widget(self.topLine1)
         self.add_widget(self.workContainerO)
         self.workContainerO.add_widget(self.workContainer)
-        # self.add_widget(Button(size_hint = (1.0, 0.01)))
 
         self.workStack = LStack()
         self.app = None
 
+        '''
+        from kivy.graphics import opengl_utils
+        print('OPENGL support:')
+        print(opengl_utils.gl_get_extensions())
+        '''
+
         # self.touches = []
 
         # beispiel zu canvas (hintergrund)
@@ -1759,10 +1826,9 @@ class LApp(App):
 
     def doSize(self, obj, val):
         mval = self.mainWindow.size
-        logging.info("LApp: size changed %s - %s (%s)" % (obj, val, mval))
-        # Clock.schedule_once(self.delayedRebuild, 0.01)
-        Clock.schedule_once(self.makeDelayedRebuild(), 0.01)
-        # self.mainWindow.rebuildContainer()
+        if (val[0] != mval[0] and val[1] != mval[1]):
+            logging.info("LApp: size changed %s - %s (%s)" % (obj, val, mval))
+            Clock.schedule_once(self.makeDelayedRebuild(), 0.01)
         pass
 
     def on_start(self):
@@ -1779,6 +1845,8 @@ class LApp(App):
         self.mainloop = self.app.mainproc()  # Einrichten
         self.mainloop.send(None)                # Spielprozess starten
         logging.info("LApp: on_start processed")
+        # Android: Request missing android permissions.
+        requestStoragePerm()
 
     def on_stop(self):
         # Achtung wird u.U. 2 mal aufgerufen !!!
@@ -1832,12 +1900,6 @@ class LApp(App):
         except Exception:
             traceback.print_exc()
             pass
-        # save comments
-        try:
-            app.saveComments()
-        except Exception:
-            traceback.print_exc()
-            pass
         logging.info("LApp: on_pause - gamesaved")
 
         logging.info("LApp: on_pause, Window.size=%s" % str(Window.size))
