@@ -23,34 +23,25 @@
 # ---------------------------------------------------------------------------#
 
 from __future__ import division
+
 import logging
+import math
 
-# PySol imports
-# from pysollib.mfxutil import Image, ImageTk
-from pysollib.kivy.LApp import LImage
-from pysollib.kivy.LApp import LImage as Image
-from pysollib.kivy.LApp import LImageItem
-from pysollib.kivy.LApp import LColorToKivy
-from pysollib.kivy.LApp import LText
-from pysollib.kivy.LApp import LRectangle
-from pysollib.kivy.LApp import LLine
-from pysollib.kivy.LApp import LAnimationManager
-
-# kivy imports
-from kivy.uix.widget import Widget
-from kivy.uix.anchorlayout import AnchorLayout
 from kivy.clock import Clock
 from kivy.graphics import Color
 from kivy.graphics import Rectangle
+from kivy.uix.anchorlayout import AnchorLayout
+from kivy.uix.widget import Widget
+
+from pysollib.kivy.LApp import LAnimationManager
+from pysollib.kivy.LApp import LColorToKivy
+from pysollib.kivy.LApp import LImage
+from pysollib.kivy.LApp import LImage as Image
+from pysollib.kivy.LApp import LImageItem
+from pysollib.kivy.LApp import LLine
+from pysollib.kivy.LApp import LRectangle
+from pysollib.kivy.LApp import LText
 
-'''
-__all__ = ['MfxCanvasGroup',
-           'MfxCanvasImage',
-           'MfxCanvasText',
-           'MfxCanvasLine',
-           'MfxCanvasRectangle',
-           'MfxCanvas']
-'''
 # ************************************************************************
 # * canvas items helpers
 # ************************************************************************
@@ -163,6 +154,17 @@ class MfxCanvasGroup():
         return None
 
 
+def cardmagnif(canvas, size):
+
+    def pyth(s):
+        return math.sqrt(s[0]*s[0]+s[1]*s[1])
+
+    cs = canvas.wmain.app.images.getSize()
+    csl = pyth(cs)
+    sl = pyth(size)
+    return csl/sl
+
+
 class MfxCanvasImage(object):
     def __init__(self, canvas, *args, **kwargs):
 
@@ -178,6 +180,9 @@ class MfxCanvasImage(object):
         self._anchor = None
         if 'anchor' in kwargs:
             self._anchor = kwargs['anchor']
+        self.hint = None
+        if 'hint' in kwargs:
+            self.hint = kwargs['hint']
 
         super(MfxCanvasImage, self).__init__()
         self.canvas = canvas
@@ -189,19 +194,17 @@ class MfxCanvasImage(object):
         if type(ed) is LImageItem:
             aimage = ed
         else:
-            if (ed.source is None):
-                image = LImage(texture=ed.texture)
-                image.size = [ed.getWidth(), ed.getHeight()]
-                aimage = LImageItem(size=image.size, group=group)
-                aimage.add_widget(image)
-                size = image.size
+            image = LImage(texture=ed.texture)
+            if self.hint == "redeal_image":
+                cm = cardmagnif(canvas, size)/3.0
+                image.size = [cm*ed.getWidth(), cm*ed.getHeight()]
             else:
-                image = LImage(texture=ed.texture)
-                # image = LImage(source=ed.source)
                 image.size = [ed.getWidth(), ed.getHeight()]
-                aimage = LImageItem(size=ed.size, group=group)
-                aimage.add_widget(image)
-                size = image.size
+
+            aimage = LImageItem(size=ed.size, group=group)
+            aimage.add_widget(image)
+            aimage.size = image.size
+            size = image.size
 
         xy = addAnchorOffset(args, self._anchor, size)
 
@@ -217,7 +220,7 @@ class MfxCanvasImage(object):
             self.addtag(group)
 
     def __del__(self):
-        print('MfxCanvasImage: __del__()')
+        print('MfxCanvasImage: __del__(%s)' % self.image)
         self.canvas.clear_widgets([self.image])
 
     def config(self, **kw):
@@ -442,7 +445,8 @@ class MfxCanvasText(object):
 class MfxCanvas(Widget):
 
     def __init__(self, wmain, *args, **kw):
-        super(MfxCanvas, self).__init__(**kw)
+        # super(MfxCanvas, self).__init__(**kw)
+        super(MfxCanvas, self).__init__()
 
         # self.tags = {}   # bei basisklasse widget (ev. nur vorläufig)
 
@@ -469,8 +473,8 @@ class MfxCanvas(Widget):
         self.r_height = None
 
         self.bindings = {}
-        self.bind(pos=self.update_widget)
-        self.bind(size=self.update_widget)
+        self.bind(pos=self.pos_update_widget)
+        self.bind(size=self.size_update_widget)
 
     def KivyToCoreP(self, pos, size, scale):
         cpos = pos
@@ -494,7 +498,7 @@ class MfxCanvas(Widget):
         return self.CoreToKivyP(cpos, csize, self.scale)
 
     def move(self, itm, dx, dy):
-            # print ('MfxCanvas: move %s %s %s' % (itm, dx, dy))
+        # print ('MfxCanvas: move %s %s %s' % (itm, dx, dy))
         scale = self.scale
         dx = scale * dx
         dy = scale * dy
@@ -506,8 +510,18 @@ class MfxCanvas(Widget):
         if self.r_height is None:
             return self.scale
 
-        scfx = self.size[0] / self.r_width
-        scfy = self.size[1] / self.r_height
+        # TBD (idee).
+        # Hier ev. einen 2ten Modus zulassen, welche das Spielfeld
+        # knapp auf die vorhandenen Karten/Anzeigeelemente bemisst.
+        # Zur Optimierung der Sichtbarkeit auf kleinen Geräten.
+        # Könnte z.B. über Doppelklick umgeschaltet werden. (Die
+        # Skalierung müsste dann allerding nach jedem Zug dem ev.
+        # veränderten Feld angepasst werden.)
+
+        wid = self.size[0]
+        hei = self.size[1]
+        scfx = wid / self.r_width
+        scfy = hei / self.r_height
 
         scf = scfx
         if (scfx < scfy):
@@ -519,10 +533,20 @@ class MfxCanvas(Widget):
 
         return scf
 
+    def pos_update_widget(self, posorobj, size):
+        # print('MfxCanvas: pos_update_widget size=(%s, %s)' %
+        #       (self.size[0], self.size[1]))
+        self.update_widget(posorobj, size)
+
+    def size_update_widget(self, posorobj, size):
+        # print('MfxCanvas: size_update_widget size=(%s, %s)' %
+        #       (self.size[0], self.size[1]))
+        self.update_widget(posorobj, size)
+
     def update_widget(self, posorobj, size):
 
-        print('MfxCanvas: update_widget size=(%s, %s)' %
-              (self.size[0], self.size[1]))
+        # print('MfxCanvas: update_widget size=(%s, %s)' %
+        #       (self.size[0], self.size[1]))
 
         # Update Skalierungsparameter
 
@@ -692,21 +716,6 @@ class MfxCanvas(Widget):
         # self.size[1] = height
         return
 
-        # print 'setInitialSize:', width, height
-        if self.preview:
-            self.config(width=width, height=height,
-                        scrollregion=(0, 0, width, height))
-        else:
-            # add margins
-            # dx, dy = 40, 40
-            dx, dy = self.xmargin, self.ymargin
-            self.config(width=dx + width + dx, height=dy + height + dy,
-                        scrollregion=(-dx, -dy, width + dx, height + dy))
-
-    #
-    #
-    #
-
     # delete all CanvasItems, but keep the background and top tiles
     def deleteAllItems(self):
         print('MfxCanvas: deleteAllItems')
@@ -845,14 +854,14 @@ class MfxCanvas(Widget):
         pass
 
     def addtag(self, tag, b, c):
-            # print ('Canvas: addtag %s %s %s' % (tag, b, c))
-            # self.tags[c] = tag
-            # self.tags.append(tag)
+        # print ('Canvas: addtag %s %s %s' % (tag, b, c))
+        # self.tags[c] = tag
+        # self.tags.append(tag)
         pass
 
     def delete(self, tag):
-            # print ('MfxCanvas: delete tag=%s' % tag)
-            # y = self.yy
+        # print ('MfxCanvas: delete tag=%s' % tag)
+        # y = self.yy
         pass
 
     def update_idletasks(self):
