@@ -21,28 +21,26 @@
 #
 # ---------------------------------------------------------------------------#
 
-# imports
 import os
 import sys
-# import htmllib
-import pysollib.htmllib2 as htmllib
-import formatter
-
-# PySol imports
-from pysollib.mygettext import _
-from pysollib.mfxutil import Struct, openURL
-from pysollib.settings import TITLE
-from pysollib.kivy.LApp import LTopLevel
-from pysollib.kivy.LApp import LScrollView
-from pysollib.kivy.LApp import LPopCommander
-from pysollib.kivy.LApp import get_platform
-from pysollib.pysoltk import MfxMessageDialog
 
 from kivy.uix.boxlayout import BoxLayout
-from kivy.uix.label import Label
 from kivy.uix.button import Button
+from kivy.uix.label import Label
 
-REMOTE_PROTOCOLS = ("ftp:", "gopher:", "http:", "mailto:", "news:", "telnet:")
+import pysollib.formatter
+import pysollib.htmllib2 as htmllib
+from pysollib.kivy.LApp import LPopCommander
+from pysollib.kivy.LApp import LScrollView
+from pysollib.kivy.LApp import LTopLevel
+from pysollib.kivy.LApp import get_platform
+from pysollib.mfxutil import Struct, openURL
+from pysollib.mygettext import _
+from pysollib.pysoltk import MfxMessageDialog
+from pysollib.settings import TITLE
+
+REMOTE_PROTOCOLS = ("ftp:", "gopher:", "http:", "https:", "mailto:", "news:",
+                    "telnet:")
 
 # ************************************************************************
 # *
@@ -87,9 +85,9 @@ def cmp2(a, b):
     return (a > b) - (a < b)
 
 
-class tkHTMLWriter(formatter.NullWriter):
+class tkHTMLWriter(pysollib.formatter.NullWriter):
     def __init__(self, text, viewer, app):
-        formatter.NullWriter.__init__(self)
+        pysollib.formatter.NullWriter.__init__(self)
 
         self.text = text
         self.viewer = viewer
@@ -409,7 +407,7 @@ class HTMLViewer:
         # self.defcursor = 'xterm'
         self.handcursor = "hand2"
 
-        self.title = "Browser"
+        self.title = _("Browser")
         self.window = None
         self.running = False
 
@@ -434,11 +432,11 @@ class HTMLViewer:
         #   BoxLayout(orientation='horizontal', size_hint=(1.0, 0.1))
 
         # create buttons
-        self.homeButton = HTMLButton(text="Index", on_release=self.goHome)
-        self.backButton = HTMLButton(text="Back", on_release=self.goBack)
+        self.homeButton = HTMLButton(text=_("Index"), on_release=self.goHome)
+        self.backButton = HTMLButton(text=_("Back"), on_release=self.goBack)
         self.forwardButton = HTMLButton(
-            text="Forward", on_release=self.goForward)
-        self.closeButton = HTMLButton(text="Close", on_release=self.goHome)
+            text=_("Forward"), on_release=self.goForward)
+        self.closeButton = HTMLButton(text=_("Close"), on_release=self.goHome)
 
         '''
         buttonline.add_widget(self.homeButton)
@@ -633,7 +631,7 @@ class HTMLViewer:
         # self.images = {}
         self.text.textbuffer = ''
         writer = tkHTMLWriter(self.text, self, self.app)
-        fmt = formatter.AbstractFormatter(writer)
+        fmt = pysollib.formatter.AbstractFormatter(writer)
         parser = tkHTMLParser(fmt)
         parser.feed(data)
         parser.close()
@@ -686,7 +684,8 @@ class HTMLViewer:
             self.display(self.home, relpath=0)
 
     def errorDialog(self, msg):
-        MfxMessageDialog(self.parent, title=TITLE + " HTML Problem",
+        MfxMessageDialog(self.parent,
+                         title=_("%s HTML Problem") % TITLE,
                          text=msg,
                          # bitmap="warning"
                          # FIXME: this interp don't have images
