@@ -22,10 +22,10 @@
 import os
 
 # PySol imports
-from pysollib.util import IMAGE_EXTENSIONS
-from pysollib.settings import TITLE
-from pysollib.winsystems import TkSettings
 from pysollib.mygettext import _, n_
+from pysollib.settings import TITLE
+from pysollib.util import IMAGE_EXTENSIONS
+from pysollib.winsystems import TkSettings
 
 # ************************************************************************
 # *
@@ -198,12 +198,13 @@ class PysolToolbarTk(BoxLayout):
             (n_("Redo"),     self.mRedo,      _("Redo last move")),
             (n_("Autodrop"), self.mDrop,      _("Auto drop cards")),
             (n_("Shuffle"),  self.mShuffle,   _("Shuffle tiles")),
+            (n_("Hint"),     self.mHint,      _("Hint")),
             (n_("Pause"),    self.mPause,     _("Pause game")),
             (None,           None,            None),
             # (n_("Statistics"), self.mPlayerStats, _("View statistics")),
             (n_("Rules"),    self.mHelpRules, _("Rules for this game")),
             (None,           None,            None),
-            (n_("Quit"),     self.mHoldAndQuit,      _("Quit ") + TITLE),
+            (n_("Quit"),     self.mHoldAndQuit,      _("Quit %s") % TITLE),
         ):
             if label is None:
                 # sep = self._createSeparator()
