@@ -26,21 +26,23 @@
 import os
 import re
 
+from findcarddialog import connect_game_find_card_dialog, \
+        destroy_find_card_dialog
+
 import gtk
 from gtk import gdk
 
-# PySol imports
-from pysollib.mygettext import _
 from pysollib.gamedb import GI
+from pysollib.mygettext import _
 from pysollib.settings import TITLE
 
-# toolkit imports
-from soundoptionsdialog import SoundOptionsDialog
 from selectcardset import SelectCardsetDialogWithPreview
-from selecttile import SelectTileDialogWithPreview
+
 from selectgame import SelectGameDialogWithPreview
-from findcarddialog import connect_game_find_card_dialog, \
-        destroy_find_card_dialog
+
+from selecttile import SelectTileDialogWithPreview
+
+from soundoptionsdialog import SoundOptionsDialog
 
 
 def ltk2gtk(s):
@@ -456,12 +458,12 @@ class PysolMenubarTk:
     def _createSelectMenu(self, games, menu):
         assert isinstance(menu, gtk.Menu)
         self._addPopularGamesMenu(games, menu)
-        for l, d in (
+        for label, d in (
             (ltk2gtk('&French games'),   GI.SELECT_GAME_BY_TYPE),
             (ltk2gtk('&Oriental games'), GI.SELECT_ORIENTAL_GAME_BY_TYPE),
             (ltk2gtk('&Special games'),  GI.SELECT_SPECIAL_GAME_BY_TYPE),
                 ):
-            self._addGamesByType(games, menu, l, d)
+            self._addGamesByType(games, menu, label, d)
         self._addMahjonggGamesMenu(games, menu)
         sep = gtk.SeparatorMenuItem()
         menu.add(sep)
@@ -609,13 +611,11 @@ class PysolMenubarTk:
         filename = self.game.filename
         if not filename:
             filename = self.app.getGameSaveName(self.game.id)
-            if os.name == 'posix':
-                filename = filename + '-' + self.game.getGameNumber(format=0)
-            elif os.path.supports_unicode_filenames:  # new in python 2.3
-                filename = filename + '-' + self.game.getGameNumber(format=0)
+            if os.name == 'posix' or os.path.supports_unicode_filenames:
+                filename += '-' + self.game.getGameNumber(format=0)
             else:
-                filename = filename + '-01'
-            filename = filename + '.pso'
+                filename += '-01'
+            filename += '.pso'
         idir, ifile = os.path.split(os.path.normpath(filename))
         if not idir:
             idir = self.app.dn.savegames
@@ -685,8 +685,11 @@ class PysolMenubarTk:
                 tile = self.app.tabletile_manager.get(0)
                 tile.color = d.key
                 self.app.setTile(0)
-            elif d.key > 0 and d.key != self.app.tabletile_index:
-                self.app.setTile(d.key)
+            elif d.key > 0 and (d.key != self.app.tabletile_index or
+                                d.preview_scaling !=
+                                self.app.opt.tabletile_scale_method):
+                self.app.setTile(d.key, scaling=d.preview_scaling)
+            self.app.opt.tabletile_scale_method = d.preview_scaling
 
     def mSelectCardsetDialog(self, *event):
         if self._cancelDrag(break_pause=False):
