@@ -21,15 +21,14 @@
 #
 # ---------------------------------------------------------------------------##
 
-# imports
-from six.moves import range
 import time
 
-# PySol imports
-from pysollib.mfxutil import format_time
 from pysollib.gamedb import GI
+from pysollib.mfxutil import format_time
 from pysollib.mygettext import _
 
+from six.moves import range
+
 # ************************************************************************
 # *
 # ************************************************************************
@@ -42,8 +41,8 @@ class PysolStatsFormatter:
                 _("Played"),
                 _("Won"),
                 _("Lost"),
-                _('Playing time'),
-                _('Moves'),
+                _('Avg. win time'),
+                _('Avg. win moves'),
                 _("% won"))
 
     def getStatResults(self, player, sort_by='name'):
@@ -173,7 +172,7 @@ class FileStatsFormatter(PysolStatsFormatter):
         self.file = file
 
     def p(self, s):
-        self.file.write(s.encode('utf-8'))
+        self.file.write(s)
 
     def nl(self, count=1):
         self.p("\n" * count)
@@ -200,7 +199,7 @@ class FileStatsFormatter(PysolStatsFormatter):
     def writeStats(self, player, sort_by='name'):
         if player is None:
             player = _('Demo')
-        header = _("Statistics for ") + player
+        header = _("Statistics for %(player)s") % {'player': player}
         self.writeHeader(header, 62)
         header = self.getStatHeader()
         self.pstats(*header)
@@ -210,7 +209,8 @@ class FileStatsFormatter(PysolStatsFormatter):
             self.pstats(gameid=gameid, *result)
         self.nl()
         total, played, won, lost, time, moves, perc = self.getStatSummary()
-        self.pstats(_("Total (%d out of %d games)") % (played, total),
+        self.pstats(_("Total (%(played)d out of %(total)d games)") %
+                    {'played': played, 'total': total},
                     won+lost, won, lost, time, moves, perc)
         self.nl(2)
         return played
@@ -232,14 +232,14 @@ class FileStatsFormatter(PysolStatsFormatter):
     def writeFullLog(self, player):
         if player is None:
             player = _('Demo')
-        header = _("Full log for ") + player
+        header = _("Full log for %(player)s") % {'player': player}
         prev_games = self.app.stats.prev_games.get(player)
         return self.writeLog(player, header, prev_games)
 
     def writeSessionLog(self, player):
         if player is None:
             player = _('Demo')
-        header = _("Session log for ") + player
+        header = _("Session log for %(player)s") % {'player': player}
         prev_games = self.app.stats.session_games.get(player)
         return self.writeLog(player, header, prev_games)
 
@@ -282,7 +282,7 @@ class ProgressionFormatter:
             t = list(t)+[0, 0, 0, -1, -1, -1]
         return list(time.localtime(time.mktime(tuple(t))))
 
-    def getResults(self, interval, all_games=True):
+    def getResults(self, interval, all_games=True, date_format='%d.%m'):
         if all_games:
             results = self.all_results
         else:
@@ -293,7 +293,7 @@ class ProgressionFormatter:
             lt = self.norm_time(t)
             marks = None
             delta = 1
-            format = '%d.%m'
+            format = date_format
         elif interval == 'month':
             tt = t[:]
             t[1] -= 1
@@ -304,7 +304,7 @@ class ProgressionFormatter:
             tt[2] -= 10
             marks.append(self.norm_time(tt)[:3])
             delta = 1
-            format = '%d.%m'
+            format = date_format
         elif interval == 'year':
             tt = t[:]
             t[0] -= 1
@@ -314,7 +314,7 @@ class ProgressionFormatter:
                 tt[1] -= 2
                 marks.append(self.norm_time(tt)[:3])
             delta = 7
-            format = '%d.%m.%y'
+            format = date_format
         else:                           # all
             tt = t[:]
             tt[1] -= 1
@@ -339,7 +339,7 @@ class ProgressionFormatter:
                 t = self.norm_time(t)
                 marks.append(t[:3])
             delta = 7
-            format = '%d.%m.%y'
+            format = date_format
 
         res = []
         ct = list(time.localtime())
